#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// istanbul ignore file
const fast_deep_equal_1 = __importDefault(require("fast-deep-equal"));
const fs_extra_1 = require("fs-extra");
const json5_1 = __importDefault(require("json5"));
const app_strings_1 = require("./config/app-strings");
const file_1 = require("./config/file");
const massage_1 = require("./config/massage");
const migration_1 = require("./config/migration");
const validation_1 = require("./config/validation");
const logger_1 = require("./logger");
/* eslint-disable no-console */
let returnVal = 0;
async function validate(desc, config, isPreset = false) {
    const { isMigrated, migratedConfig } = migration_1.migrateConfig(config);
    if (isMigrated) {
        logger_1.logger.warn({
            oldConfig: config,
            newConfig: migratedConfig,
        }, 'Config migration necessary');
    }
    const massagedConfig = massage_1.massageConfig(migratedConfig);
    const res = await validation_1.validateConfig(massagedConfig, isPreset);
    if (res.errors.length) {
        logger_1.logger.error({ errors: res.errors }, `${desc} contains errors`);
        returnVal = 1;
    }
    if (res.warnings.length) {
        logger_1.logger.warn({ warnings: res.warnings }, `${desc} contains warnings`);
        returnVal = 1;
    }
}
(async () => {
    var _a;
    for (const file of app_strings_1.configFileNames.filter((name) => name !== 'package.json')) {
        try {
            const rawContent = fs_extra_1.readFileSync(file, 'utf8');
            logger_1.logger.info(`Validating ${file}`);
            try {
                let jsonContent;
                if (file.endsWith('.json5')) {
                    jsonContent = json5_1.default.parse(rawContent);
                }
                else {
                    jsonContent = JSON.parse(rawContent);
                }
                await validate(file, jsonContent);
            }
            catch (err) {
                logger_1.logger.info({ err }, `${file} is not valid Renovate config`);
                returnVal = 1;
            }
        }
        catch (err) {
            // file does not exist
        }
    }
    try {
        const pkgJson = JSON.parse(fs_extra_1.readFileSync('package.json', 'utf8'));
        if (pkgJson.renovate) {
            logger_1.logger.info(`Validating package.json > renovate`);
            await validate('package.json > renovate', pkgJson.renovate);
        }
        if (pkgJson['renovate-config']) {
            logger_1.logger.info(`Validating package.json > renovate-config`);
            for (const presetConfig of Object.values(pkgJson['renovate-config'])) {
                await validate('package.json > renovate-config', presetConfig, true);
            }
        }
    }
    catch (err) {
        // ignore
    }
    try {
        const fileConfig = file_1.getConfig(process.env);
        if (!fast_deep_equal_1.default(fileConfig, {})) {
            const file = (_a = process.env.RENOVATE_CONFIG_FILE) !== null && _a !== void 0 ? _a : 'config.js';
            logger_1.logger.info(`Validating ${file}`);
            try {
                await validate(file, fileConfig);
            }
            catch (err) {
                logger_1.logger.error({ err }, `${file} is not valid Renovate config`);
                returnVal = 1;
            }
        }
    }
    catch (err) {
        // ignore
    }
    if (returnVal !== 0) {
        process.exit(returnVal);
    }
    logger_1.logger.info('Config validated successfully');
})().catch((e) => {
    console.error(e);
    process.exit(99);
});
//# sourceMappingURL=config-validator.js.map