"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDigest = exports.getReleases = exports.defaultRegistryUrls = exports.registryStrategy = exports.id = void 0;
const packageCache = __importStar(require("../../util/cache/package"));
const bitbucket_1 = require("../../util/http/bitbucket");
const url_1 = require("../../util/url");
const bitbucketHttp = new bitbucket_1.BitbucketHttp();
exports.id = 'bitbucket-tags';
exports.registryStrategy = 'first';
exports.defaultRegistryUrls = ['https://bitbucket.org'];
function getRegistryURL(registryUrl) {
    // fallback to default API endpoint if custom not provided
    return registryUrl !== null && registryUrl !== void 0 ? registryUrl : exports.defaultRegistryUrls[0];
}
const cacheNamespace = 'datasource-bitbucket';
function getCacheKey(registryUrl, repo, type) {
    return `${getRegistryURL(registryUrl)}:${repo}:${type}`;
}
// getReleases fetches list of tags for the repository
async function getReleases({ registryUrl, lookupName: repo, }) {
    const cacheKey = getCacheKey(registryUrl, repo, 'tags');
    const cachedResult = await packageCache.get(cacheNamespace, cacheKey);
    // istanbul ignore if
    if (cachedResult) {
        return cachedResult;
    }
    const url = `/2.0/repositories/${repo}/refs/tags`;
    const bitbucketTags = (await bitbucketHttp.getJson(url)).body;
    const dependency = {
        sourceUrl: `${url_1.ensureTrailingSlash(getRegistryURL(registryUrl))}${repo}`,
        releases: null,
    };
    dependency.releases = bitbucketTags.values.map(({ name, target }) => ({
        version: name,
        gitRef: name,
        releaseTimestamp: target === null || target === void 0 ? void 0 : target.date,
    }));
    const cacheMinutes = 10;
    await packageCache.set(cacheNamespace, cacheKey, dependency, cacheMinutes);
    return dependency;
}
exports.getReleases = getReleases;
// getTagCommit fetched the commit has for specified tag
async function getTagCommit(registryUrl, repo, tag) {
    const cacheKey = getCacheKey(registryUrl, repo, `tag-${tag}`);
    const cachedResult = await packageCache.get(cacheNamespace, cacheKey);
    // istanbul ignore if
    if (cachedResult) {
        return cachedResult;
    }
    const url = `/2.0/repositories/${repo}/refs/tags/${tag}`;
    const bitbucketTag = (await bitbucketHttp.getJson(url)).body;
    const hash = bitbucketTag.target.hash;
    const cacheMinutes = 10;
    await packageCache.set(cacheNamespace, cacheKey, hash, cacheMinutes);
    return hash;
}
// getDigest fetched the latest commit for repository main branch
// however, if newValue is provided, then getTagCommit is called
async function getDigest({ lookupName: repo, registryUrl }, newValue) {
    if (newValue === null || newValue === void 0 ? void 0 : newValue.length) {
        return getTagCommit(registryUrl, repo, newValue);
    }
    const cacheKey = getCacheKey(registryUrl, repo, 'digest');
    const cachedResult = await packageCache.get(cacheNamespace, cacheKey);
    // istanbul ignore if
    if (cachedResult) {
        return cachedResult;
    }
    const branchCacheKey = getCacheKey(registryUrl, repo, 'mainbranch');
    let mainBranch = await packageCache.get(cacheNamespace, branchCacheKey);
    if (!mainBranch) {
        mainBranch = (await bitbucketHttp.getJson(`/2.0/repositories/${repo}`)).body.mainbranch.name;
        await packageCache.set(cacheNamespace, branchCacheKey, mainBranch, 60);
    }
    const url = `/2.0/repositories/${repo}/commits/${mainBranch}`;
    const bitbucketCommits = (await bitbucketHttp.getJson(url)).body;
    if (bitbucketCommits.values.length === 0) {
        return null;
    }
    const latestCommit = bitbucketCommits.values[0].hash;
    const cacheMinutes = 10;
    await packageCache.set(cacheNamespace, cacheKey, latestCommit, cacheMinutes);
    return latestCommit;
}
exports.getDigest = getDigest;
//# sourceMappingURL=index.js.map