"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractPackageFile = void 0;
const is_1 = __importDefault(require("@sindresorhus/is"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const logger_1 = require("../../logger");
const extract_1 = require("../dockerfile/extract");
function extractPackageFile(content) {
    var _a;
    const deps = [];
    try {
        const doc = js_yaml_1.default.safeLoad(content);
        if ((doc === null || doc === void 0 ? void 0 : doc.steps) && is_1.default.array(doc.steps)) {
            for (const step of doc.steps) {
                if (step.name) {
                    const dep = extract_1.getDep(step.name);
                    logger_1.logger.trace({
                        depName: dep.depName,
                        currentValue: dep.currentValue,
                        currentDigest: dep.currentDigest,
                    }, 'Cloud Build docker image');
                    deps.push(dep);
                }
            }
        }
    }
    catch (err) /* istanbul ignore next */ {
        if ((_a = err.stack) === null || _a === void 0 ? void 0 : _a.startsWith('YAMLException:')) {
            logger_1.logger.debug({ err }, 'YAML exception extracting Docker images from a Cloud Build configuration file.');
        }
        else {
            logger_1.logger.warn({ err }, 'Error extracting Docker images from a Cloud Build configuration file.');
        }
    }
    if (!deps.length) {
        return null;
    }
    return { deps };
}
exports.extractPackageFile = extractPackageFile;
//# sourceMappingURL=extract.js.map