"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractPackageFile = void 0;
const is_1 = __importDefault(require("@sindresorhus/is"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const datasourceHelm = __importStar(require("../../datasource/helm"));
const logger_1 = require("../../logger");
const types_1 = require("../../types");
function extractPackageFile(content, fileName, config) {
    let deps = [];
    let doc;
    try {
        doc = js_yaml_1.default.safeLoad(content, { json: true });
    }
    catch (err) {
        logger_1.logger.debug({ fileName }, 'Failed to parse helm requirements.yaml');
        return null;
    }
    if (!(doc && is_1.default.array(doc.dependencies))) {
        logger_1.logger.debug({ fileName }, 'requirements.yaml has no dependencies');
        return null;
    }
    deps = doc.dependencies.map((dep) => {
        const res = {
            depName: dep.name,
            currentValue: dep.version,
        };
        if (!res.depName) {
            res.skipReason = types_1.SkipReason.InvalidName;
            return res;
        }
        if (!res.currentValue) {
            res.skipReason = types_1.SkipReason.InvalidVersion;
            return res;
        }
        if (!dep.repository) {
            res.skipReason = types_1.SkipReason.NoRepository;
            return res;
        }
        res.registryUrls = [dep.repository];
        if (dep.repository.startsWith('@')) {
            const repoWithAtRemoved = dep.repository.slice(1);
            const alias = config.aliases[repoWithAtRemoved];
            if (alias) {
                res.registryUrls = [alias];
                return res;
            }
            res.skipReason = types_1.SkipReason.PlaceholderUrl;
        }
        else {
            try {
                const url = new URL(dep.repository);
                if (url.protocol === 'file:') {
                    res.skipReason = types_1.SkipReason.LocalDependency;
                }
            }
            catch (err) {
                logger_1.logger.debug({ err }, 'Error parsing url');
                res.skipReason = types_1.SkipReason.InvalidUrl;
            }
        }
        return res;
    });
    const res = {
        deps,
        datasource: datasourceHelm.id,
    };
    return res;
}
exports.extractPackageFile = extractPackageFile;
//# sourceMappingURL=extract.js.map