"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findDepConstraints = void 0;
const npm_1 = require("../../../../versioning/npm");
// Finds all parent dependencies for a given depName@currentVersion
function findDepConstraints(packageJson, lockEntry, depName, currentVersion, newVersion, parentDepName) {
    var _a, _b;
    let parents = [];
    let packageJsonConstraint = (_a = packageJson.dependencies) === null || _a === void 0 ? void 0 : _a[depName];
    if (packageJsonConstraint) {
        parents.push({
            depType: 'dependencies',
            constraint: packageJsonConstraint,
        });
    }
    packageJsonConstraint = (_b = packageJson.devDependencies) === null || _b === void 0 ? void 0 : _b[depName];
    if (packageJsonConstraint) {
        parents.push({
            depType: 'devDependencies',
            constraint: packageJsonConstraint,
        });
    }
    const { dependencies, requires, version } = lockEntry;
    if (parentDepName && requires) {
        const constraint = requires[depName];
        if (constraint && npm_1.api.matches(currentVersion, constraint)) {
            if (constraint === currentVersion) {
                // Workaround for old versions of npm which wrote the exact version in requires instead of the constraint
                requires[depName] = newVersion;
            }
            parents.push({
                parentDepName,
                parentVersion: version,
                constraint,
            });
        }
    }
    if (dependencies) {
        for (const [packageName, dependency] of Object.entries(dependencies)) {
            parents = parents.concat(findDepConstraints(packageJson, dependency, depName, currentVersion, newVersion, packageName));
        }
    }
    // dedupe
    const res = [];
    for (const req of parents) {
        const reqStringified = JSON.stringify(req);
        if (!res.find((i) => JSON.stringify(i) === reqStringified)) {
            res.push(req);
        }
    }
    return res;
}
exports.findDepConstraints = findDepConstraints;
//# sourceMappingURL=dep-constraints.js.map