"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getVulnerabilityAlerts = exports.deleteLabel = exports.addReviewers = exports.addAssignees = exports.getIssueList = exports.ensureIssueClosing = exports.ensureIssue = exports.findIssue = exports.massageMarkdown = exports.mergePr = exports.setBranchStatus = exports.ensureCommentRemoval = exports.ensureComment = exports.updatePr = exports.createPr = exports.getBranchStatus = exports.getBranchStatusCheck = exports.getBranchPr = exports.findPr = exports.getPr = exports.getPrList = exports.getRepoForceRebase = exports.initRepo = exports.getJsonFile = exports.getRepos = exports.initPlatform = void 0;
const is_1 = __importDefault(require("@sindresorhus/is"));
const GitInterfaces_1 = require("azure-devops-node-api/interfaces/GitInterfaces");
const delay_1 = __importDefault(require("delay"));
const error_messages_1 = require("../../constants/error-messages");
const platforms_1 = require("../../constants/platforms");
const logger_1 = require("../../logger");
const types_1 = require("../../types");
const git = __importStar(require("../../util/git"));
const hostRules = __importStar(require("../../util/host-rules"));
const sanitize_1 = require("../../util/sanitize");
const url_1 = require("../../util/url");
const pr_body_1 = require("../utils/pr-body");
const azureApi = __importStar(require("./azure-got-wrapper"));
const azureHelper = __importStar(require("./azure-helper"));
const util_1 = require("./util");
let config = {};
const defaults = {
    hostType: platforms_1.PLATFORM_TYPE_AZURE,
};
function initPlatform({ endpoint, token, username, password, }) {
    if (!endpoint) {
        throw new Error('Init: You must configure an Azure DevOps endpoint');
    }
    if (!token && !(username && password)) {
        throw new Error('Init: You must configure an Azure DevOps token, or a username and password');
    }
    // TODO: Add a connection check that endpoint/token combination are valid
    const res = {
        endpoint: url_1.ensureTrailingSlash(endpoint),
    };
    defaults.endpoint = res.endpoint;
    azureApi.setEndpoint(res.endpoint);
    const platformConfig = {
        endpoint: defaults.endpoint,
    };
    return Promise.resolve(platformConfig);
}
exports.initPlatform = initPlatform;
async function getRepos() {
    logger_1.logger.debug('Autodiscovering Azure DevOps repositories');
    const azureApiGit = await azureApi.gitApi();
    const repos = await azureApiGit.getRepositories();
    return repos.map((repo) => `${repo.project.name}/${repo.name}`);
}
exports.getRepos = getRepos;
async function getJsonFile(fileName) {
    try {
        const json = await azureHelper.getFile(config.repoId, fileName, config.defaultBranch);
        return JSON.parse(json);
    }
    catch (err) {
        return null;
    }
}
exports.getJsonFile = getJsonFile;
async function initRepo({ repository, localDir, cloneSubmodules, }) {
    var _a, _b;
    logger_1.logger.debug(`initRepo("${repository}")`);
    config = { repository };
    const azureApiGit = await azureApi.gitApi();
    const repos = await azureApiGit.getRepositories();
    const names = azureHelper.getProjectAndRepo(repository);
    const repo = repos.filter((c) => c.name.toLowerCase() === names.repo.toLowerCase() &&
        c.project.name.toLowerCase() === names.project.toLowerCase())[0];
    logger_1.logger.debug({ repositoryDetails: repo }, 'Repository details');
    // istanbul ignore if
    if (!repo.defaultBranch) {
        logger_1.logger.debug('Repo is empty');
        throw new Error(error_messages_1.REPOSITORY_EMPTY);
    }
    config.repoId = repo.id;
    config.project = repo.project.name;
    config.owner = '?owner?';
    logger_1.logger.debug(`${repository} owner = ${config.owner}`);
    const defaultBranch = repo.defaultBranch.replace('refs/heads/', '');
    config.defaultBranch = defaultBranch;
    logger_1.logger.debug(`${repository} default branch = ${defaultBranch}`);
    config.defaultMergeMethod = await azureHelper.getMergeMethod(repo.id, names.project);
    config.mergeMethods = {};
    config.repoForceRebase = false;
    const [projectName, repoName] = repository.split('/');
    const opts = hostRules.find({
        hostType: defaults.hostType,
        url: defaults.endpoint,
    });
    const manualUrl = defaults.endpoint +
        `${encodeURIComponent(projectName)}/_git/${encodeURIComponent(repoName)}`;
    const url = repo.remoteUrl || manualUrl;
    await git.initRepo({
        ...config,
        localDir,
        url,
        extraCloneOpts: azureHelper.getStorageExtraCloneOpts(opts),
        gitAuthorName: (_a = global.gitAuthor) === null || _a === void 0 ? void 0 : _a.name,
        gitAuthorEmail: (_b = global.gitAuthor) === null || _b === void 0 ? void 0 : _b.email,
        cloneSubmodules,
    });
    const repoConfig = {
        defaultBranch,
        isFork: false,
    };
    return repoConfig;
}
exports.initRepo = initRepo;
function getRepoForceRebase() {
    return Promise.resolve(config.repoForceRebase === true);
}
exports.getRepoForceRebase = getRepoForceRebase;
async function getPrList() {
    logger_1.logger.debug('getPrList()');
    if (!config.prList) {
        const azureApiGit = await azureApi.gitApi();
        let prs = [];
        let fetchedPrs;
        let skip = 0;
        do {
            fetchedPrs = await azureApiGit.getPullRequests(config.repoId, { status: 4 }, config.project, 0, skip, 100);
            prs = prs.concat(fetchedPrs);
            skip += 100;
        } while (fetchedPrs.length > 0);
        config.prList = prs.map(util_1.getRenovatePRFormat);
        logger_1.logger.debug({ length: config.prList.length }, 'Retrieved Pull Requests');
    }
    return config.prList;
}
exports.getPrList = getPrList;
async function getPr(pullRequestId) {
    logger_1.logger.debug(`getPr(${pullRequestId})`);
    if (!pullRequestId) {
        return null;
    }
    const azurePr = (await getPrList()).find((item) => item.number === pullRequestId);
    if (!azurePr) {
        return null;
    }
    const azureApiGit = await azureApi.gitApi();
    const labels = await azureApiGit.getPullRequestLabels(config.repoId, pullRequestId);
    azurePr.labels = labels
        .filter((label) => label.active)
        .map((label) => label.name);
    azurePr.hasReviewers = is_1.default.nonEmptyArray(azurePr.reviewers);
    return azurePr;
}
exports.getPr = getPr;
async function findPr({ branchName, prTitle, state = types_1.PrState.All, }) {
    let prsFiltered = [];
    try {
        const prs = await getPrList();
        prsFiltered = prs.filter((item) => item.sourceRefName === util_1.getNewBranchName(branchName));
        if (prTitle) {
            prsFiltered = prsFiltered.filter((item) => item.title === prTitle);
        }
        switch (state) {
            case types_1.PrState.All:
                // no more filter needed, we can go further...
                break;
            case types_1.PrState.NotOpen:
                prsFiltered = prsFiltered.filter((item) => item.state !== types_1.PrState.Open);
                break;
            default:
                prsFiltered = prsFiltered.filter((item) => item.state === state);
                break;
        }
    }
    catch (err) {
        logger_1.logger.error({ err }, 'findPr error');
    }
    if (prsFiltered.length === 0) {
        return null;
    }
    return prsFiltered[0];
}
exports.findPr = findPr;
async function getBranchPr(branchName) {
    logger_1.logger.debug(`getBranchPr(${branchName})`);
    const existingPr = await findPr({
        branchName,
        state: types_1.PrState.Open,
    });
    return existingPr ? getPr(existingPr.number) : null;
}
exports.getBranchPr = getBranchPr;
async function getStatusCheck(branchName) {
    const azureApiGit = await azureApi.gitApi();
    const branch = await azureApiGit.getBranch(config.repoId, util_1.getBranchNameWithoutRefsheadsPrefix(branchName));
    // only grab the latest statuses, it will group any by context
    return azureApiGit.getStatuses(branch.commit.commitId, config.repoId, undefined, undefined, undefined, true);
}
const azureToRenovateStatusMapping = {
    [GitInterfaces_1.GitStatusState.Succeeded]: types_1.BranchStatus.green,
    [GitInterfaces_1.GitStatusState.NotApplicable]: types_1.BranchStatus.green,
    [GitInterfaces_1.GitStatusState.NotSet]: types_1.BranchStatus.yellow,
    [GitInterfaces_1.GitStatusState.Pending]: types_1.BranchStatus.yellow,
    [GitInterfaces_1.GitStatusState.Error]: types_1.BranchStatus.red,
    [GitInterfaces_1.GitStatusState.Failed]: types_1.BranchStatus.red,
};
async function getBranchStatusCheck(branchName, context) {
    const res = await getStatusCheck(branchName);
    for (const check of res) {
        if (util_1.getGitStatusContextCombinedName(check.context) === context) {
            return azureToRenovateStatusMapping[check.state] || types_1.BranchStatus.yellow;
        }
    }
    return null;
}
exports.getBranchStatusCheck = getBranchStatusCheck;
async function getBranchStatus(branchName, requiredStatusChecks) {
    logger_1.logger.debug(`getBranchStatus(${branchName})`);
    if (!requiredStatusChecks) {
        // null means disable status checks, so it always succeeds
        return types_1.BranchStatus.green;
    }
    if (requiredStatusChecks.length) {
        // This is Unsupported
        logger_1.logger.warn({ requiredStatusChecks }, `Unsupported requiredStatusChecks`);
        return types_1.BranchStatus.red;
    }
    const statuses = await getStatusCheck(branchName);
    logger_1.logger.debug({ branch: branchName, statuses }, 'branch status check result');
    if (!statuses.length) {
        logger_1.logger.debug('empty branch status check result = returning "pending"');
        return types_1.BranchStatus.yellow;
    }
    const noOfFailures = statuses.filter((status) => status.state === GitInterfaces_1.GitStatusState.Error ||
        status.state === GitInterfaces_1.GitStatusState.Failed).length;
    if (noOfFailures) {
        return types_1.BranchStatus.red;
    }
    const noOfPending = statuses.filter((status) => status.state === GitInterfaces_1.GitStatusState.NotSet ||
        status.state === GitInterfaces_1.GitStatusState.Pending).length;
    if (noOfPending) {
        return types_1.BranchStatus.yellow;
    }
    return types_1.BranchStatus.green;
}
exports.getBranchStatus = getBranchStatus;
async function createPr({ sourceBranch, targetBranch, prTitle: title, prBody: body, labels, draftPR = false, platformOptions, }) {
    var _a;
    const sourceRefName = util_1.getNewBranchName(sourceBranch);
    const targetRefName = util_1.getNewBranchName(targetBranch);
    const description = azureHelper.max4000Chars(sanitize_1.sanitize(body));
    const azureApiGit = await azureApi.gitApi();
    const workItemRefs = [
        {
            id: (_a = platformOptions === null || platformOptions === void 0 ? void 0 : platformOptions.azureWorkItemId) === null || _a === void 0 ? void 0 : _a.toString(),
        },
    ];
    let pr = await azureApiGit.createPullRequest({
        sourceRefName,
        targetRefName,
        title,
        description,
        workItemRefs,
        isDraft: draftPR,
    }, config.repoId);
    if (platformOptions === null || platformOptions === void 0 ? void 0 : platformOptions.azureAutoComplete) {
        pr = await azureApiGit.updatePullRequest({
            autoCompleteSetBy: {
                id: pr.createdBy.id,
            },
            completionOptions: {
                mergeStrategy: config.defaultMergeMethod,
                deleteSourceBranch: true,
            },
        }, config.repoId, pr.pullRequestId);
    }
    await Promise.all(labels.map((label) => azureApiGit.createPullRequestLabel({
        name: label,
    }, config.repoId, pr.pullRequestId)));
    return util_1.getRenovatePRFormat(pr);
}
exports.createPr = createPr;
async function updatePr({ number: prNo, prTitle: title, prBody: body, state, }) {
    logger_1.logger.debug(`updatePr(${prNo}, ${title}, body)`);
    const azureApiGit = await azureApi.gitApi();
    const objToUpdate = {
        title,
    };
    if (body) {
        objToUpdate.description = azureHelper.max4000Chars(sanitize_1.sanitize(body));
    }
    if (state === types_1.PrState.Open) {
        await azureApiGit.updatePullRequest({ status: GitInterfaces_1.PullRequestStatus.Active }, config.repoId, prNo);
    }
    else if (state === types_1.PrState.Closed) {
        objToUpdate.status = GitInterfaces_1.PullRequestStatus.Abandoned;
    }
    await azureApiGit.updatePullRequest(objToUpdate, config.repoId, prNo);
}
exports.updatePr = updatePr;
async function ensureComment({ number, topic, content, }) {
    logger_1.logger.debug(`ensureComment(${number}, ${topic}, content)`);
    const header = topic ? `### ${topic}\n\n` : '';
    const body = `${header}${sanitize_1.sanitize(content)}`;
    const azureApiGit = await azureApi.gitApi();
    const threads = await azureApiGit.getThreads(config.repoId, number);
    let threadIdFound = null;
    let commentIdFound = null;
    let commentNeedsUpdating = false;
    threads.forEach((thread) => {
        const firstCommentContent = thread.comments[0].content;
        if ((topic && (firstCommentContent === null || firstCommentContent === void 0 ? void 0 : firstCommentContent.startsWith(header))) ||
            (!topic && firstCommentContent === body)) {
            threadIdFound = thread.id;
            commentIdFound = thread.comments[0].id;
            commentNeedsUpdating = firstCommentContent !== body;
        }
    });
    if (!threadIdFound) {
        await azureApiGit.createThread({
            comments: [{ content: body, commentType: 1, parentCommentId: 0 }],
            status: 1,
        }, config.repoId, number);
        logger_1.logger.info({ repository: config.repository, issueNo: number, topic }, 'Comment added');
    }
    else if (commentNeedsUpdating) {
        await azureApiGit.updateComment({
            content: body,
        }, config.repoId, number, threadIdFound, commentIdFound);
        logger_1.logger.debug({ repository: config.repository, issueNo: number, topic }, 'Comment updated');
    }
    else {
        logger_1.logger.debug({ repository: config.repository, issueNo: number, topic }, 'Comment is already update-to-date');
    }
    return true;
}
exports.ensureComment = ensureComment;
async function ensureCommentRemoval({ number: issueNo, topic, content, }) {
    var _a, _b;
    logger_1.logger.debug(`Ensuring comment "${topic || content}" in #${issueNo} is removed`);
    const azureApiGit = await azureApi.gitApi();
    const threads = await azureApiGit.getThreads(config.repoId, issueNo);
    const byTopic = (thread) => thread.comments[0].content.startsWith(`### ${topic}\n\n`);
    const byContent = (thread) => thread.comments[0].content.trim() === content;
    let threadIdFound = null;
    if (topic) {
        threadIdFound = (_a = threads.find(byTopic)) === null || _a === void 0 ? void 0 : _a.id;
    }
    else if (content) {
        threadIdFound = (_b = threads.find(byContent)) === null || _b === void 0 ? void 0 : _b.id;
    }
    if (threadIdFound) {
        await azureApiGit.updateThread({
            status: 4, // close
        }, config.repoId, issueNo, threadIdFound);
    }
}
exports.ensureCommentRemoval = ensureCommentRemoval;
const renovateToAzureStatusMapping = {
    [types_1.BranchStatus.green]: [GitInterfaces_1.GitStatusState.Succeeded],
    [types_1.BranchStatus.green]: GitInterfaces_1.GitStatusState.Succeeded,
    [types_1.BranchStatus.yellow]: GitInterfaces_1.GitStatusState.Pending,
    [types_1.BranchStatus.red]: GitInterfaces_1.GitStatusState.Failed,
};
async function setBranchStatus({ branchName, context, description, state, url: targetUrl, }) {
    logger_1.logger.debug(`setBranchStatus(${branchName}, ${context}, ${description}, ${state}, ${targetUrl})`);
    const azureApiGit = await azureApi.gitApi();
    const branch = await azureApiGit.getBranch(config.repoId, util_1.getBranchNameWithoutRefsheadsPrefix(branchName));
    const statusToCreate = {
        description,
        context: util_1.getGitStatusContextFromCombinedName(context),
        state: renovateToAzureStatusMapping[state],
        targetUrl,
    };
    await azureApiGit.createCommitStatus(statusToCreate, branch.commit.commitId, config.repoId);
    logger_1.logger.trace(`Created commit status of ${state} on branch ${branchName}`);
}
exports.setBranchStatus = setBranchStatus;
async function mergePr(pullRequestId, branchName) {
    var _a;
    logger_1.logger.debug(`mergePr(${pullRequestId}, ${branchName})`);
    const azureApiGit = await azureApi.gitApi();
    let pr = await azureApiGit.getPullRequestById(pullRequestId, config.project);
    const mergeMethod = (_a = config.mergeMethods[pr.targetRefName]) !== null && _a !== void 0 ? _a : (config.mergeMethods[pr.targetRefName] = await azureHelper.getMergeMethod(config.repoId, config.project, pr.targetRefName));
    const objToUpdate = {
        status: GitInterfaces_1.PullRequestStatus.Completed,
        lastMergeSourceCommit: pr.lastMergeSourceCommit,
        completionOptions: {
            mergeStrategy: mergeMethod,
            deleteSourceBranch: true,
        },
    };
    logger_1.logger.trace(`Updating PR ${pullRequestId} to status ${GitInterfaces_1.PullRequestStatus.Completed} (${GitInterfaces_1.PullRequestStatus[GitInterfaces_1.PullRequestStatus.Completed]}) with lastMergeSourceCommit ${pr.lastMergeSourceCommit.commitId} using mergeStrategy ${mergeMethod} (${GitInterfaces_1.GitPullRequestMergeStrategy[mergeMethod]})`);
    try {
        const response = await azureApiGit.updatePullRequest(objToUpdate, config.repoId, pullRequestId);
        let retries = 0;
        let isClosed = response.status === GitInterfaces_1.PullRequestStatus.Completed;
        while (!isClosed && retries < 5) {
            retries += 1;
            const sleepMs = retries * 1000;
            logger_1.logger.trace({ pullRequestId, status: pr.status, retries }, `Updated PR to closed status but change has not taken effect yet. Retrying...`);
            await delay_1.default(sleepMs);
            pr = await azureApiGit.getPullRequestById(pullRequestId, config.project);
            isClosed = pr.status === GitInterfaces_1.PullRequestStatus.Completed;
        }
        if (!isClosed) {
            logger_1.logger.warn({ pullRequestId, status: pr.status }, `Expected PR to have status ${GitInterfaces_1.PullRequestStatus[GitInterfaces_1.PullRequestStatus.Completed]}, however it is ${GitInterfaces_1.PullRequestStatus[pr.status]}.`);
        }
        return true;
    }
    catch (err) {
        logger_1.logger.debug({ err }, 'Failed to set the PR as completed.');
        return false;
    }
}
exports.mergePr = mergePr;
function massageMarkdown(input) {
    // Remove any HTML we use
    return pr_body_1.smartTruncate(input, 4000)
        .replace('you tick the rebase/retry checkbox', 'rename PR to start with "rebase!"')
        .replace(new RegExp(`\n---\n\n.*?<!-- rebase-check -->.*?\n`), '')
        .replace('<summary>', '**')
        .replace('</summary>', '**')
        .replace('<details>', '')
        .replace('</details>', '');
}
exports.massageMarkdown = massageMarkdown;
/* istanbul ignore next */
function findIssue() {
    logger_1.logger.warn(`findIssue() is not implemented`);
    return null;
}
exports.findIssue = findIssue;
/* istanbul ignore next */
function ensureIssue() {
    logger_1.logger.warn(`ensureIssue() is not implemented`);
    return Promise.resolve(null);
}
exports.ensureIssue = ensureIssue;
/* istanbul ignore next */
function ensureIssueClosing() {
    return Promise.resolve();
}
exports.ensureIssueClosing = ensureIssueClosing;
/* istanbul ignore next */
function getIssueList() {
    logger_1.logger.debug(`getIssueList()`);
    // TODO: Needs implementation
    return Promise.resolve([]);
}
exports.getIssueList = getIssueList;
async function getUserIds(users) {
    const azureApiGit = await azureApi.gitApi();
    const azureApiCore = await azureApi.coreApi();
    const repos = await azureApiGit.getRepositories();
    const repo = repos.filter((c) => c.id === config.repoId)[0];
    const teams = await azureApiCore.getTeams(repo.project.id);
    const members = await Promise.all(teams.map(async (t) => 
    /* eslint-disable no-return-await,@typescript-eslint/return-await */
    await azureApiCore.getTeamMembersWithExtendedProperties(repo.project.id, t.id)));
    const ids = [];
    members.forEach((listMembers) => {
        listMembers.forEach((m) => {
            users.forEach((r) => {
                if (r.toLowerCase() === m.identity.displayName.toLowerCase() ||
                    r.toLowerCase() === m.identity.uniqueName.toLowerCase()) {
                    if (ids.filter((c) => c.id === m.identity.id).length === 0) {
                        ids.push({ id: m.identity.id, name: r });
                    }
                }
            });
        });
    });
    teams.forEach((t) => {
        users.forEach((r) => {
            if (r.toLowerCase() === t.name.toLowerCase()) {
                if (ids.filter((c) => c.id === t.id).length === 0) {
                    ids.push({ id: t.id, name: r });
                }
            }
        });
    });
    return ids;
}
/**
 *
 * @param {number} issueNo
 * @param {string[]} assignees
 */
async function addAssignees(issueNo, assignees) {
    logger_1.logger.trace(`addAssignees(${issueNo}, [${assignees.join(', ')}])`);
    const ids = await getUserIds(assignees);
    await ensureComment({
        number: issueNo,
        topic: 'Add Assignees',
        content: ids.map((a) => `@<${a.id}>`).join(', '),
    });
}
exports.addAssignees = addAssignees;
/**
 *
 * @param {number} prNo
 * @param {string[]} reviewers
 */
async function addReviewers(prNo, reviewers) {
    logger_1.logger.trace(`addReviewers(${prNo}, [${reviewers.join(', ')}])`);
    const azureApiGit = await azureApi.gitApi();
    const ids = await getUserIds(reviewers);
    await Promise.all(ids.map(async (obj) => {
        await azureApiGit.createPullRequestReviewer({}, config.repoId, prNo, obj.id);
        logger_1.logger.debug(`Reviewer added: ${obj.name}`);
    }));
}
exports.addReviewers = addReviewers;
async function deleteLabel(prNumber, label) {
    logger_1.logger.debug(`Deleting label ${label} from #${prNumber}`);
    const azureApiGit = await azureApi.gitApi();
    await azureApiGit.deletePullRequestLabels(config.repoId, prNumber, label);
}
exports.deleteLabel = deleteLabel;
function getVulnerabilityAlerts() {
    return Promise.resolve([]);
}
exports.getVulnerabilityAlerts = getVulnerabilityAlerts;
//# sourceMappingURL=index.js.map