"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNewValue = void 0;
const semver_1 = require("semver");
const semver_utils_1 = require("semver-utils");
const logger_1 = require("../../logger");
function getNewValue({ currentValue, rangeStrategy, currentVersion, newVersion, }) {
    var _a, _b;
    if (rangeStrategy === 'pin' || semver_1.valid(currentValue)) {
        return newVersion;
    }
    if (rangeStrategy === 'update-lockfile') {
        if (semver_1.satisfies(newVersion, currentValue)) {
            return currentValue;
        }
        return getNewValue({
            currentValue,
            rangeStrategy: 'replace',
            currentVersion,
            newVersion,
        });
    }
    const parsedRange = semver_utils_1.parseRange(currentValue);
    const element = parsedRange[parsedRange.length - 1];
    if (rangeStrategy === 'widen') {
        const newValue = getNewValue({
            currentValue,
            rangeStrategy: 'replace',
            currentVersion,
            newVersion,
        });
        if ((_a = element.operator) === null || _a === void 0 ? void 0 : _a.startsWith('<')) {
            // TODO fix this
            const splitCurrent = currentValue.split(element.operator);
            splitCurrent.pop();
            return splitCurrent.join(element.operator) + newValue;
        }
        if (parsedRange.length > 1) {
            const previousElement = parsedRange[parsedRange.length - 2];
            if (previousElement.operator === '-') {
                const splitCurrent = currentValue.split('-');
                splitCurrent.pop();
                return splitCurrent.join('-') + '- ' + newValue;
            }
            if ((_b = element.operator) === null || _b === void 0 ? void 0 : _b.startsWith('>')) {
                logger_1.logger.warn(`Complex ranges ending in greater than are not supported`);
                return null;
            }
        }
        return `${currentValue} || ${newValue}`;
    }
    const toVersionMajor = semver_1.major(newVersion);
    const toVersionMinor = semver_1.minor(newVersion);
    const toVersionPatch = semver_1.patch(newVersion);
    const suffix = semver_1.prerelease(newVersion) ? '-' + semver_1.prerelease(newVersion)[0] : '';
    // Simple range
    if (rangeStrategy === 'bump') {
        if (parsedRange.length === 1) {
            if (!element.operator) {
                return getNewValue({
                    currentValue,
                    rangeStrategy: 'replace',
                    currentVersion,
                    newVersion,
                });
            }
            if (element.operator === '^') {
                const split = currentValue.split('.');
                if (suffix.length) {
                    return `^${newVersion}`;
                }
                if (split.length === 1) {
                    // ^4
                    return `^${toVersionMajor}`;
                }
                if (split.length === 2) {
                    // ^4.1
                    return `^${toVersionMajor}.${toVersionMinor}`;
                }
                return `^${newVersion}`;
            }
            if (element.operator === '~') {
                const split = currentValue.split('.');
                if (suffix.length) {
                    return `~${newVersion}`;
                }
                if (split.length === 1) {
                    // ~4
                    return `~${toVersionMajor}`;
                }
                if (split.length === 2) {
                    // ~4.1
                    return `~${toVersionMajor}.${toVersionMinor}`;
                }
                return `~${newVersion}`;
            }
            if (element.operator === '=') {
                return `=${newVersion}`;
            }
            if (element.operator === '>=') {
                return currentValue.includes('>= ')
                    ? `>= ${newVersion}`
                    : `>=${newVersion}`;
            }
            if (element.operator.startsWith('<')) {
                return currentValue;
            }
        }
        else {
            const newRange = semver_utils_1.parseRange(currentValue);
            const versions = newRange.map((x) => {
                const subRange = x.semver;
                const bumpedSubRange = getNewValue({
                    currentValue: subRange,
                    rangeStrategy: 'bump',
                    currentVersion,
                    newVersion,
                });
                if (semver_1.satisfies(newVersion, bumpedSubRange)) {
                    return bumpedSubRange;
                }
                return getNewValue({
                    currentValue: subRange,
                    rangeStrategy: 'replace',
                    currentVersion,
                    newVersion,
                });
            });
            return versions.filter((x) => x !== null && x !== '').join(' ');
        }
        logger_1.logger.debug('Unsupported range type for rangeStrategy=bump: ' + currentValue);
        return null;
    }
    if (element.operator === '~>') {
        return `~> ${toVersionMajor}.${toVersionMinor}.0`;
    }
    if (element.operator === '^') {
        if (suffix.length || !currentVersion) {
            return `^${toVersionMajor}.${toVersionMinor}.${toVersionPatch}${suffix}`;
        }
        if (toVersionMajor === semver_1.major(currentVersion)) {
            if (toVersionMajor === 0) {
                if (toVersionMinor === 0) {
                    return `^${newVersion}`;
                }
                return `^${toVersionMajor}.${toVersionMinor}.0`;
            }
            return `^${newVersion}`;
        }
        return `^${toVersionMajor}.0.0`;
    }
    if (element.operator === '=') {
        return `=${newVersion}`;
    }
    if (element.operator === '~') {
        if (suffix.length) {
            return `~${toVersionMajor}.${toVersionMinor}.${toVersionPatch}${suffix}`;
        }
        return `~${toVersionMajor}.${toVersionMinor}.0`;
    }
    if (element.operator === '<=') {
        let res;
        if (element.patch || suffix.length) {
            res = `<=${newVersion}`;
        }
        else if (element.minor) {
            res = `<=${toVersionMajor}.${toVersionMinor}`;
        }
        else {
            res = `<=${toVersionMajor}`;
        }
        if (currentValue.includes('<= ')) {
            res = res.replace('<=', '<= ');
        }
        return res;
    }
    if (element.operator === '<') {
        let res;
        if (currentValue.endsWith('.0.0')) {
            const newMajor = toVersionMajor + 1;
            res = `<${newMajor}.0.0`;
        }
        else if (element.patch) {
            res = `<${semver_1.inc(newVersion, 'patch')}`;
        }
        else if (element.minor) {
            res = `<${toVersionMajor}.${toVersionMinor + 1}`;
        }
        else {
            res = `<${toVersionMajor + 1}`;
        }
        if (currentValue.includes('< ')) {
            res = res.replace(/</g, '< ');
        }
        return res;
    }
    if (!element.operator) {
        if (element.minor) {
            if (element.minor === 'x') {
                return `${toVersionMajor}.x`;
            }
            if (element.minor === '*') {
                return `${toVersionMajor}.*`;
            }
            if (element.patch === 'x') {
                return `${toVersionMajor}.${toVersionMinor}.x`;
            }
            if (element.patch === '*') {
                return `${toVersionMajor}.${toVersionMinor}.*`;
            }
            return `${toVersionMajor}.${toVersionMinor}`;
        }
        return `${toVersionMajor}`;
    }
    return newVersion;
}
exports.getNewValue = getNewValue;
//# sourceMappingURL=range.js.map