"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renovateRepository = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const admin_1 = require("../../config/admin");
const logger_1 = require("../../logger");
const fs_1 = require("../../util/fs");
const queue = __importStar(require("../../util/http/queue"));
const split_1 = require("../../util/split");
const cache_1 = require("./cache");
const dependency_dashboard_1 = require("./dependency-dashboard");
const error_1 = __importDefault(require("./error"));
const finalise_1 = require("./finalise");
const init_1 = require("./init");
const pr_1 = require("./onboarding/pr");
const process_1 = require("./process");
const result_1 = require("./result");
const stats_1 = require("./stats");
let renovateVersion = 'unknown';
try {
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    renovateVersion = require('../../../package.json').version; // eslint-disable-line global-require
}
catch (err) /* istanbul ignore next */ {
    logger_1.logger.debug({ err }, 'Error getting renovate version');
}
// istanbul ignore next
async function renovateRepository(repoConfig) {
    split_1.splitInit();
    let config = { ...repoConfig };
    admin_1.setAdminConfig(config);
    logger_1.setMeta({ repository: config.repository });
    logger_1.logger.info({ renovateVersion }, 'Repository started');
    logger_1.logger.trace({ config });
    let repoResult;
    queue.clear();
    try {
        await fs_extra_1.default.ensureDir(config.localDir);
        logger_1.logger.debug('Using localDir: ' + config.localDir);
        config = await init_1.initRepo(config);
        split_1.addSplit('init');
        const { branches, branchList, packageFiles } = await process_1.extractDependencies(config);
        await pr_1.ensureOnboardingPr(config, packageFiles, branches);
        const res = await process_1.updateRepo(config, branches);
        split_1.addSplit('update');
        await cache_1.setBranchCache(branches);
        if (res !== 'automerged') {
            await dependency_dashboard_1.ensureMasterIssue(config, branches);
        }
        await finalise_1.finaliseRepo(config, branchList);
        repoResult = result_1.processResult(config, res);
    }
    catch (err) /* istanbul ignore next */ {
        logger_1.setMeta({ repository: config.repository });
        const errorRes = await error_1.default(config, err);
        repoResult = result_1.processResult(config, errorRes);
    }
    if (config.localDir && !config.persistRepoData) {
        try {
            await fs_1.deleteLocalFile('.');
        }
        catch (err) /* istanbul ignore if */ {
            logger_1.logger.warn({ err }, 'localDir deletion error');
        }
    }
    try {
        await fs_extra_1.default.remove(fs_1.privateCacheDir());
    }
    catch (err) /* istanbul ignore if */ {
        logger_1.logger.warn({ err }, 'privateCacheDir deletion error');
    }
    const splits = split_1.getSplits();
    logger_1.logger.debug(splits, 'Repository timing splits (milliseconds)');
    stats_1.printRequestStats();
    logger_1.logger.info({ durationMs: splits.total }, 'Repository finished');
    return repoResult;
}
exports.renovateRepository = renovateRepository;
//# sourceMappingURL=index.js.map