"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCurrentVersion = void 0;
const logger_1 = require("../../../../logger");
function getCurrentVersion(config, versioning, rangeStrategy, latestVersion, allVersions) {
    const { currentValue, lockedVersion } = config;
    if (versioning.isVersion(currentValue)) {
        return currentValue;
    }
    if (versioning.isSingleVersion(currentValue)) {
        return currentValue.replace(/=/g, '').trim();
    }
    logger_1.logger.trace(`currentValue ${currentValue} is range`);
    let useVersions = allVersions.filter((v) => versioning.matches(v, currentValue));
    if (latestVersion && versioning.matches(latestVersion, currentValue)) {
        useVersions = useVersions.filter((v) => !versioning.isGreaterThan(v, latestVersion));
    }
    if (rangeStrategy === 'pin') {
        return (lockedVersion ||
            versioning.getSatisfyingVersion(useVersions, currentValue));
    }
    if (rangeStrategy === 'bump') {
        // Use the lowest version in the current range
        return versioning.minSatisfyingVersion(useVersions, currentValue);
    }
    // Use the highest version in the current range
    return versioning.getSatisfyingVersion(useVersions, currentValue);
}
exports.getCurrentVersion = getCurrentVersion;
//# sourceMappingURL=current.js.map