"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectVulnerabilityAlerts = void 0;
const error_messages_1 = require("../../../constants/error-messages");
const datasourceMaven = __importStar(require("../../../datasource/maven"));
const datasourceNpm = __importStar(require("../../../datasource/npm"));
const datasourceNuget = __importStar(require("../../../datasource/nuget"));
const datasourcePypi = __importStar(require("../../../datasource/pypi"));
const datasourceRubygems = __importStar(require("../../../datasource/rubygems"));
const logger_1 = require("../../../logger");
const platform_1 = require("../../../platform");
const markdown_1 = require("../../../util/markdown");
const allVersioning = __importStar(require("../../../versioning"));
const mavenVersioning = __importStar(require("../../../versioning/maven"));
const npmVersioning = __importStar(require("../../../versioning/npm"));
const pep440Versioning = __importStar(require("../../../versioning/pep440"));
const rubyVersioning = __importStar(require("../../../versioning/ruby"));
const semverVersioning = __importStar(require("../../../versioning/semver"));
async function detectVulnerabilityAlerts(input) {
    var _a, _b, _c, _d;
    if (!(input === null || input === void 0 ? void 0 : input.vulnerabilityAlerts)) {
        return input;
    }
    if (input.vulnerabilityAlerts.enabled === false) {
        logger_1.logger.debug('Vulnerability alerts are disabled');
        return input;
    }
    const alerts = await platform_1.platform.getVulnerabilityAlerts();
    if (!alerts.length) {
        logger_1.logger.debug('No vulnerability alerts found');
        if (input.vulnerabilityAlertsOnly) {
            throw new Error(error_messages_1.NO_VULNERABILITY_ALERTS);
        }
        return input;
    }
    const config = { ...input };
    const versionings = {
        maven: mavenVersioning.id,
        npm: npmVersioning.id,
        nuget: semverVersioning.id,
        pip_requirements: pep440Versioning.id,
        rubygems: rubyVersioning.id,
    };
    const combinedAlerts = {};
    for (const alert of alerts) {
        try {
            if (alert.dismissReason) {
                continue; // eslint-disable-line no-continue
            }
            if (!alert.securityVulnerability.firstPatchedVersion) {
                logger_1.logger.debug({ alert }, 'Vulnerability alert has no firstPatchedVersion - skipping');
                continue; // eslint-disable-line no-continue
            }
            const datasourceMapping = {
                MAVEN: datasourceMaven.id,
                NPM: datasourceNpm.id,
                NUGET: datasourceNuget.id,
                PIP: datasourcePypi.id,
                RUBYGEMS: datasourceRubygems.id,
            };
            const datasource = datasourceMapping[alert.securityVulnerability.package.ecosystem];
            const depName = alert.securityVulnerability.package.name;
            const fileName = alert.vulnerableManifestPath;
            const fileType = alert.vulnerableManifestFilename;
            const firstPatchedVersion = alert.securityVulnerability.firstPatchedVersion.identifier;
            const advisory = alert.securityAdvisory;
            let { vulnerableRequirements } = alert;
            // istanbul ignore if
            if (!vulnerableRequirements.length) {
                if (datasource === datasourceMaven.id) {
                    vulnerableRequirements = `(,${firstPatchedVersion})`;
                }
                else {
                    vulnerableRequirements = `< ${firstPatchedVersion}`;
                }
            }
            combinedAlerts[fileName] || (combinedAlerts[fileName] = {});
            (_a = combinedAlerts[fileName])[datasource] || (_a[datasource] = {});
            (_b = combinedAlerts[fileName][datasource])[depName] || (_b[depName] = {});
            (_c = combinedAlerts[fileName][datasource][depName])[vulnerableRequirements] || (_c[vulnerableRequirements] = {
                advisories: [],
            });
            const alertDetails = combinedAlerts[fileName][datasource][depName][vulnerableRequirements];
            alertDetails.advisories.push(advisory);
            const version = allVersioning.get(versionings[datasource]);
            if (version.isVersion(firstPatchedVersion)) {
                if (!alertDetails.firstPatchedVersion ||
                    version.isGreaterThan(firstPatchedVersion, alertDetails.firstPatchedVersion)) {
                    alertDetails.firstPatchedVersion = firstPatchedVersion;
                }
            }
            else {
                logger_1.logger.debug('Invalid firstPatchedVersion: ' + firstPatchedVersion);
            }
            alertDetails.fileType = fileType;
        }
        catch (err) {
            logger_1.logger.warn({ err }, 'Error parsing vulnerability alert');
        }
    }
    const alertPackageRules = [];
    config.remediations = {};
    for (const [fileName, files] of Object.entries(combinedAlerts)) {
        for (const [datasource, dependencies] of Object.entries(files)) {
            for (const [depName, currentValues] of Object.entries(dependencies)) {
                for (const [matchCurrentVersion, val] of Object.entries(currentValues)) {
                    let prBodyNotes = [];
                    try {
                        prBodyNotes = ['### GitHub Vulnerability Alerts'].concat(val.advisories.map((advisory) => {
                            let content = '#### ';
                            let heading;
                            if (advisory.identifiers.some((id) => id.type === 'CVE')) {
                                heading = advisory.identifiers
                                    .filter((id) => id.type === 'CVE')
                                    .map((id) => id.value)
                                    .join(' / ');
                            }
                            else {
                                heading = advisory.identifiers
                                    .map((id) => id.value)
                                    .join(' / ');
                            }
                            if (advisory.references.length) {
                                heading = `[${heading}](${advisory.references[0].url})`;
                            }
                            content += heading;
                            content += '\n\n';
                            // eslint-disable-next-line no-loop-func
                            content += markdown_1.sanitizeMarkdown(advisory.description);
                            return content;
                        }));
                    }
                    catch (err) /* istanbul ignore next */ {
                        logger_1.logger.warn({ err }, 'Error generating vulnerability PR notes');
                    }
                    const allowedVersions = datasource === datasourcePypi.id
                        ? `==${val.firstPatchedVersion}`
                        : val.firstPatchedVersion;
                    const matchRule = {
                        matchDatasources: [datasource],
                        matchPackageNames: [depName],
                        matchCurrentVersion,
                        matchFiles: [fileName],
                    };
                    const supportedRemediationFileTypes = ['package-lock.json'];
                    if (config.transitiveRemediation &&
                        supportedRemediationFileTypes.includes(val.fileType)) {
                        (_d = config.remediations)[fileName] || (_d[fileName] = []);
                        const currentVersion = matchCurrentVersion.replace('=', '').trim();
                        const newVersion = allowedVersions;
                        const remediation = {
                            datasource,
                            depName,
                            currentVersion,
                            newVersion,
                            prBodyNotes,
                        };
                        config.remediations[fileName].push(remediation);
                        // Disable the package rule as all vulnerabilities will be remediated via the lock file
                        matchRule.enabled = false;
                    }
                    else {
                        // Remediate only direct dependencies
                        Object.assign(matchRule, {
                            allowedVersions,
                            prBodyNotes,
                            isVulnerabilityAlert: true,
                            force: {
                                ...config.vulnerabilityAlerts,
                            },
                        });
                    }
                    alertPackageRules.push(matchRule);
                }
            }
        }
    }
    logger_1.logger.debug({ alertPackageRules }, 'alert package rules');
    config.packageRules = (config.packageRules || []).concat(alertPackageRules);
    return config;
}
exports.detectVulnerabilityAlerts = detectVulnerabilityAlerts;
//# sourceMappingURL=vulnerability.js.map